'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var view = require('@codemirror/view');
var state = require('@codemirror/state');
var gutter = require('@codemirror/gutter');

let relativeLineNumberGutter = new state.Compartment();
class Marker extends gutter.GutterMarker {
    constructor(text) {
        super();
        this.text = text;
    }
    toDOM() {
        return document.createTextNode(this.text);
    }
}
const absoluteLineNumberGutter = gutter.gutter({
    lineMarker: (view, line) => {
        const lineNo = view.state.doc.lineAt(line.from).number;
        const absoluteLineNo = new Marker(lineNo.toString());
        const cursorLine = view.state.doc.lineAt(view.state.selection.asSingle().ranges[0].to).number;
        if (lineNo === cursorLine) {
            return absoluteLineNo;
        }
        return null;
    },
    initialSpacer: () => {
        const spacer = new Marker("0");
        return spacer;
    },
});
function relativeLineNumbers(lineNo, state) {
    if (lineNo > state.doc.lines) {
        return lineNo.toString();
    }
    const cursorLine = state.doc.lineAt(state.selection.asSingle().ranges[0].to).number;
    if (lineNo === cursorLine) {
        return lineNo.toString();
    }
    else {
        return Math.abs(cursorLine - lineNo).toString();
    }
}
// This shows the numbers in the gutter
const showLineNumbers = relativeLineNumberGutter.of(gutter.lineNumbers({ formatNumber: relativeLineNumbers }));
// This ensures the numbers update
// when selection (cursorActivity) happens
const lineNumbersUpdateListener = view.EditorView.updateListener.of((viewUpdate) => {
    if (viewUpdate.selectionSet) {
        viewUpdate.view.dispatch({
            effects: relativeLineNumberGutter.reconfigure(gutter.lineNumbers({ formatNumber: relativeLineNumbers })),
        });
    }
});
function lineNumbersRelative() {
    return [absoluteLineNumberGutter, showLineNumbers, lineNumbersUpdateListener];
}

exports.lineNumbersRelative = lineNumbersRelative;
