import { EditorView } from '@codemirror/view';
import { Compartment } from '@codemirror/state';
import { gutter, lineNumbers, GutterMarker } from '@codemirror/gutter';

let relativeLineNumberGutter = /*@__PURE__*/new Compartment();
class Marker extends GutterMarker {
    constructor(text) {
        super();
        this.text = text;
    }
    toDOM() {
        return document.createTextNode(this.text);
    }
}
const absoluteLineNumberGutter = /*@__PURE__*/gutter({
    lineMarker: (view, line) => {
        const lineNo = view.state.doc.lineAt(line.from).number;
        const absoluteLineNo = new Marker(lineNo.toString());
        const cursorLine = view.state.doc.lineAt(view.state.selection.asSingle().ranges[0].to).number;
        if (lineNo === cursorLine) {
            return absoluteLineNo;
        }
        return null;
    },
    initialSpacer: () => {
        const spacer = new Marker("0");
        return spacer;
    },
});
function relativeLineNumbers(lineNo, state) {
    if (lineNo > state.doc.lines) {
        return lineNo.toString();
    }
    const cursorLine = state.doc.lineAt(state.selection.asSingle().ranges[0].to).number;
    if (lineNo === cursorLine) {
        return lineNo.toString();
    }
    else {
        return Math.abs(cursorLine - lineNo).toString();
    }
}
// This shows the numbers in the gutter
const showLineNumbers = /*@__PURE__*/relativeLineNumberGutter.of(/*@__PURE__*/lineNumbers({ formatNumber: relativeLineNumbers }));
// This ensures the numbers update
// when selection (cursorActivity) happens
const lineNumbersUpdateListener = /*@__PURE__*/EditorView.updateListener.of((viewUpdate) => {
    if (viewUpdate.selectionSet) {
        viewUpdate.view.dispatch({
            effects: relativeLineNumberGutter.reconfigure(lineNumbers({ formatNumber: relativeLineNumbers })),
        });
    }
});
function lineNumbersRelative() {
    return [absoluteLineNumberGutter, showLineNumbers, lineNumbersUpdateListener];
}

export { lineNumbersRelative };
